module Sec
  module Firms
    class FirmParser < XMLParser
      SEC_FIRM_DETAIL_FILE_NAME = 'primary_doc.xml'
      SEC_FIRM_URL = 'http://www.sec.gov/cgi-bin/browse-edgar?action=getcompany&CIK=:CIK_ID&output=atom'

      def initialize(cik_id)
        fetch_url = SEC_FIRM_URL.gsub(':CIK_ID', cik_id)
        xml_str = Net::HTTP.get(URI.parse(fetch_url))
        super(xml_str)
      end

      def id
        doc.xpath('//xmlns:cik').text
      end

      def slug
        Slugify.convert(name[:value], true)
      end

      def name
        default_values doc.xpath('//xmlns:conformed-name').text
      end

      def to_hash
        data = Hash.from_xml(doc.to_s).try(:[], 'feed').try(:[], 'company_info') || []
        data = Helpers::transform_hash(data) do |hash, key, value|
          hash[key.underscore] = default_values(value)
        end

        {
          id: id,
          slug: slug,
          name: name
        }.merge(data).merge(entries_attributes)
          .deep_transform_keys { |key| key.to_s.underscore }
      end

      def entries
        doc.xpath('//xmlns:entry').map do |entry|
          filing_date = Date.parse(entry.at('filing-date').text)
          accession_number = entry.at('accession-nunber').text
          [
            [filing_date, accession_number],
            entry
          ]
        end.sort_by { |value| value[0] }
      end

      private

      def updated_at
        @updated_at ||= doc.xpath('//xmlns:feed').at('updated').text
      end

      def default_values(value)
        {
          value: value,
          updated_at: updated_at
        }
      end

      def entries_attributes
        hash = {}
        details = entries.map do |entry|
          FirmEntryParser.new(
            Downloader.new(entry_url(entry[1])).content, entry[0][0]
          ).to_hash
        end

        details.each do |detail|
          hash.merge!(detail)
        end
        hash
      end

      def entry_url(entry)
        url = entry.at('filing-href').text
        url.gsub(url.split('/')[-1], SEC_FIRM_DETAIL_FILE_NAME)
      end
    end
  end
end
